@@ -24,6 +24,7 @@
 
 #include <stdlib.h>
 #include <glib/gi18n.h>
+#include <tepl/tepl.h>
 
 #include "gedit-app.h"
 #include "gedit-app-private.h"
@@ -32,7 +33,6 @@
 #include "gedit-io-error-info-bar.h"
 #include "gedit-print-job.h"
 #include "gedit-print-preview.h"
-#include "gedit-progress-info-bar.h"
 #include "gedit-debug.h"
 #include "gedit-document.h"
 #include "gedit-document-private.h"
@@ -53,7 +53,6 @@ struct _GeditTab
 	GeditViewFrame *frame;
 
 	GtkWidget *info_bar;
-	GtkWidget *info_bar_hidden;
 
 	GeditPrintJob *print_job;
 	GtkWidget *print_preview;
@@ -144,9 +143,6 @@ static void launch_loader (GTask                   *loading_task,
 
 static void launch_saver (GTask *saving_task);
 
-static void continue_loading(GTask     *loading_task);
-
-
 static SaverData *
 saver_data_new (void)
 {
@@ -584,9 +580,9 @@ document_location_notify_handler (GtkSourceFile *file,
 }
 
 static void
-document_shortname_notify_handler (GeditDocument *document,
-				   GParamSpec    *pspec,
-				   GeditTab      *tab)
+document_shortname_notify_handler (TeplFile   *file,
+				   GParamSpec *pspec,
+				   GeditTab   *tab)
 {
 	gedit_debug (DEBUG_TAB);
 
@@ -602,57 +598,33 @@ document_modified_changed (GtkTextBuffer *document,
 	g_object_notify_by_pspec (G_OBJECT (tab), properties[PROP_CAN_CLOSE]);
 }
 
+/* This function must be used carefully, and should be replaced by
+ * tepl_tab_add_info_bar() (note the *add*, not *set*).
+ * When certain infobars are set, it also configures GeditTab to be in a certain
+ * state (e.g. non-editable) and it waits a response from the infobar to restore
+ * the GeditTab state. If another infobar is set in the meantime, there will be
+ * a bug.
+ */
 static void
-set_info_bar (GeditTab        *tab,
-              GtkWidget       *info_bar,
-              GtkResponseType  default_response)
+set_info_bar (GeditTab  *tab,
+	      GtkWidget *info_bar)
 {
-	gedit_debug (DEBUG_TAB);
-
 	if (tab->info_bar == info_bar)
 	{
 		return;
 	}
 
-	if (info_bar == NULL)
+	if (tab->info_bar != NULL)
 	{
-		/* Don't destroy the old info_bar right away,
-		   we want the hide animation. */
-		if (tab->info_bar_hidden != NULL)
-		{
-			gtk_widget_destroy (tab->info_bar_hidden);
-		}
-
-		tab->info_bar_hidden = tab->info_bar;
-		gtk_widget_hide (tab->info_bar_hidden);
-
+		gtk_widget_destroy (tab->info_bar);
 		tab->info_bar = NULL;
 	}
-	else
+
+	tab->info_bar = info_bar;
+
+	if (info_bar != NULL)
 	{
-		if (tab->info_bar != NULL)
-		{
-			gedit_debug_message (DEBUG_TAB, "Replacing existing notification");
-			gtk_widget_destroy (tab->info_bar);
-		}
-
-		/* Make sure to stop a possibly still ongoing hiding animation. */
-		if (tab->info_bar_hidden != NULL)
-		{
-			gtk_widget_destroy (tab->info_bar_hidden);
-			tab->info_bar_hidden = NULL;
-		}
-
-		tab->info_bar = info_bar;
 		gtk_box_pack_start (GTK_BOX (tab), info_bar, FALSE, FALSE, 0);
-
-		/* Note this must be done after the info bar is added to the window */
-		if (default_response != GTK_RESPONSE_NONE)
-		{
-			gtk_info_bar_set_default_response (GTK_INFO_BAR (info_bar),
-			                                   default_response);
-		}
-
 		gtk_widget_show (info_bar);
 	}
 }
@@ -682,7 +654,7 @@ io_loading_error_info_bar_response (GtkWidget *info_bar,
 		case GTK_RESPONSE_OK:
 			encoding = gedit_conversion_error_info_bar_get_encoding (GTK_WIDGET (info_bar));
 
-			set_info_bar (data->tab, NULL, GTK_RESPONSE_NONE);
+			set_info_bar (data->tab, NULL);
 			gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_LOADING);
 
 			launch_loader (loading_task, encoding);
@@ -691,21 +663,13 @@ io_loading_error_info_bar_response (GtkWidget *info_bar,
 		case GTK_RESPONSE_YES:
 			/* This means that we want to edit the document anyway */
 			set_editable (data->tab, TRUE);
-			set_info_bar (data->tab, NULL, GTK_RESPONSE_NONE);
+			set_info_bar (data->tab, NULL);
 			gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_NORMAL);
 
 			g_task_return_boolean (loading_task, TRUE);
 			g_object_unref (loading_task);
 			break;
 
-
-		case GTK_RESPONSE_ACCEPT:
-			set_info_bar (data->tab, NULL, GTK_RESPONSE_NONE);
-			gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_LOADING);
-
-			continue_loading (loading_task);
-			break;
-
 		default:
 			if (location != NULL)
 			{
@@ -732,7 +696,7 @@ file_already_open_warning_info_bar_response (GtkWidget *info_bar,
 		set_editable (tab, TRUE);
 	}
 
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
@@ -744,7 +708,7 @@ load_cancelled (GtkWidget *bar,
 {
 	LoaderData *data = g_task_get_task_data (loading_task);
 
-	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (data->tab->info_bar));
+	g_return_if_fail (TEPL_IS_PROGRESS_INFO_BAR (data->tab->info_bar));
 
 	g_cancellable_cancel (g_task_get_cancellable (loading_task));
 	remove_tab (data->tab);
@@ -760,7 +724,7 @@ unrecoverable_reverting_error_info_bar_response (GtkWidget *info_bar,
 
 	gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_NORMAL);
 
-	set_info_bar (data->tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (data->tab, NULL);
 
 	view = gedit_tab_get_view (data->tab);
 	gtk_widget_grab_focus (GTK_WIDGET (view));
@@ -775,7 +739,7 @@ static void
 show_loading_info_bar (GTask *loading_task)
 {
 	LoaderData *data = g_task_get_task_data (loading_task);
-	GtkWidget *bar;
+	TeplProgressInfoBar *bar;
 	GeditDocument *doc;
 	gchar *name;
 	gchar *dirname = NULL;
@@ -803,7 +767,7 @@ show_loading_info_bar (GTask *loading_task)
 	{
 		gchar *str;
 
-		str = gedit_utils_str_middle_truncate (name, MAX_MSG_LENGTH);
+		str = tepl_utils_str_middle_truncate (name, MAX_MSG_LENGTH);
 		g_free (name);
 		name = str;
 	}
@@ -822,8 +786,8 @@ show_loading_info_bar (GTask *loading_task)
 			 * we have a title long 99 + 20, but I think it's a rare enough
 			 * case to be acceptable. It's justa darn title afterall :)
 			 */
-			dirname = gedit_utils_str_middle_truncate (str,
-								   MAX (20, MAX_MSG_LENGTH - len));
+			dirname = tepl_utils_str_middle_truncate (str,
+								  MAX (20, MAX_MSG_LENGTH - len));
 			g_free (str);
 		}
 	}
@@ -848,7 +812,7 @@ show_loading_info_bar (GTask *loading_task)
 			msg = g_strdup_printf (_("Reverting %s"), name_markup);
 		}
 
-		bar = gedit_progress_info_bar_new ("document-revert", msg, TRUE);
+		bar = tepl_progress_info_bar_new ("document-revert", msg, TRUE);
 	}
 	else
 	{
@@ -868,7 +832,7 @@ show_loading_info_bar (GTask *loading_task)
 			msg = g_strdup_printf (_("Loading %s"), name_markup);
 		}
 
-		bar = gedit_progress_info_bar_new ("document-open", msg, TRUE);
+		bar = tepl_progress_info_bar_new ("document-open", msg, TRUE);
 	}
 
 	g_signal_connect_object (bar,
@@ -877,7 +841,7 @@ show_loading_info_bar (GTask *loading_task)
 				 loading_task,
 				 0);
 
-	set_info_bar (data->tab, bar, GTK_RESPONSE_NONE);
+	set_info_bar (data->tab, GTK_WIDGET (bar));
 
 	g_free (msg);
 	g_free (name);
@@ -889,7 +853,7 @@ static void
 show_saving_info_bar (GTask *saving_task)
 {
 	GeditTab *tab = g_task_get_source_object (saving_task);
-	GtkWidget *bar;
+	TeplProgressInfoBar *bar;
 	GeditDocument *doc;
 	gchar *short_name;
 	gchar *from;
@@ -917,7 +881,7 @@ show_saving_info_bar (GTask *saving_task)
 	 */
 	if (len > MAX_MSG_LENGTH)
 	{
-		from = gedit_utils_str_middle_truncate (short_name, MAX_MSG_LENGTH);
+		from = tepl_utils_str_middle_truncate (short_name, MAX_MSG_LENGTH);
 		g_free (short_name);
 	}
 	else
@@ -931,7 +895,7 @@ show_saving_info_bar (GTask *saving_task)
 
 		from = short_name;
 		to = g_file_get_parse_name (location);
-		str = gedit_utils_str_middle_truncate (to, MAX (20, MAX_MSG_LENGTH - len));
+		str = tepl_utils_str_middle_truncate (to, MAX (20, MAX_MSG_LENGTH - len));
 		g_free (to);
 
 		to = str;
@@ -953,9 +917,9 @@ show_saving_info_bar (GTask *saving_task)
 		msg = g_strdup_printf (_("Saving %s"), from_markup);
 	}
 
-	bar = gedit_progress_info_bar_new ("document-save", msg, FALSE);
+	bar = tepl_progress_info_bar_new ("document-save", msg, FALSE);
 
-	set_info_bar (tab, bar, GTK_RESPONSE_NONE);
+	set_info_bar (tab, GTK_WIDGET (bar));
 
 	g_free (msg);
 	g_free (to);
@@ -968,7 +932,7 @@ info_bar_set_progress (GeditTab *tab,
 		       goffset   size,
 		       goffset   total_size)
 {
-	GeditProgressInfoBar *progress_info_bar;
+	TeplProgressInfoBar *progress_info_bar;
 
 	if (tab->info_bar == NULL)
 	{
@@ -977,23 +941,23 @@ info_bar_set_progress (GeditTab *tab,
 
 	gedit_debug_message (DEBUG_TAB, "%" G_GOFFSET_FORMAT "/%" G_GOFFSET_FORMAT, size, total_size);
 
-	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->info_bar));
+	g_return_if_fail (TEPL_IS_PROGRESS_INFO_BAR (tab->info_bar));
 
-	progress_info_bar = GEDIT_PROGRESS_INFO_BAR (tab->info_bar);
+	progress_info_bar = TEPL_PROGRESS_INFO_BAR (tab->info_bar);
 
 	if (total_size != 0)
 	{
 		gdouble frac = (gdouble)size / (gdouble)total_size;
 
-		gedit_progress_info_bar_set_fraction (progress_info_bar, frac);
+		tepl_progress_info_bar_set_fraction (progress_info_bar, frac);
 	}
 	else if (size != 0)
 	{
-		gedit_progress_info_bar_pulse (progress_info_bar);
+		tepl_progress_info_bar_pulse (progress_info_bar);
 	}
 	else
 	{
-		gedit_progress_info_bar_set_fraction (progress_info_bar, 0);
+		tepl_progress_info_bar_set_fraction (progress_info_bar, 0);
 	}
 }
 
@@ -1052,7 +1016,7 @@ scroll_timeout_cb (GeditTab *tab)
 	GeditView *view;
 
 	view = gedit_tab_get_view (tab);
-	gedit_view_scroll_to_cursor (view);
+	tepl_view_scroll_to_cursor (TEPL_VIEW (view));
 
 	tab->scroll_timeout = 0;
 	return G_SOURCE_REMOVE;
@@ -1090,7 +1054,7 @@ unrecoverable_saving_error_info_bar_response (GtkWidget *info_bar,
 
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	view = gedit_tab_get_view (tab);
 	gtk_widget_grab_focus (GTK_WIDGET (view));
@@ -1139,7 +1103,7 @@ invalid_character_info_bar_response (GtkWidget *info_bar,
 		SaverData *data = g_task_get_task_data (saving_task);
 		GtkSourceFileSaverFlags save_flags;
 
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 
 		/* Don't bug the user again with this... */
 		tab->save_flags |= GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_INVALID_CHARS;
@@ -1158,9 +1122,9 @@ invalid_character_info_bar_response (GtkWidget *info_bar,
 }
 
 static void
-no_backup_error_info_bar_response (GtkWidget *info_bar,
-				   gint       response_id,
-				   GTask     *saving_task)
+cant_create_backup_error_info_bar_response (GtkWidget *info_bar,
+					    gint       response_id,
+					    GTask     *saving_task)
 {
 	if (response_id == GTK_RESPONSE_YES)
 	{
@@ -1168,7 +1132,7 @@ no_backup_error_info_bar_response (GtkWidget *info_bar,
 		SaverData *data = g_task_get_task_data (saving_task);
 		GtkSourceFileSaverFlags save_flags;
 
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 
 		data->force_no_backup = TRUE;
 		save_flags = gtk_source_file_saver_get_flags (data->saver);
@@ -1194,7 +1158,7 @@ externally_modified_error_info_bar_response (GtkWidget *info_bar,
 		SaverData *data = g_task_get_task_data (saving_task);
 		GtkSourceFileSaverFlags save_flags;
 
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 
 		/* ignore_modification_time should not be persisted in save
 		 * flags across saves (i.e. tab->save_flags is not modified).
@@ -1223,7 +1187,7 @@ recoverable_saving_error_info_bar_response (GtkWidget *info_bar,
 		SaverData *data = g_task_get_task_data (saving_task);
 		const GtkSourceEncoding *encoding;
 
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 
 		encoding = gedit_conversion_error_info_bar_get_encoding (GTK_WIDGET (info_bar));
 		g_return_if_fail (encoding != NULL);
@@ -1244,7 +1208,7 @@ externally_modified_notification_info_bar_response (GtkWidget *info_bar,
 {
 	GeditView *view;
 
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	view = gedit_tab_get_view (tab);
 
@@ -1266,7 +1230,7 @@ externally_modified_notification_info_bar_response (GtkWidget *info_bar,
 static void
 display_externally_modified_notification (GeditTab *tab)
 {
-	GtkWidget *info_bar;
+	TeplInfoBar *info_bar;
 	GeditDocument *doc;
 	GtkSourceFile *file;
 	GFile *location;
@@ -1280,9 +1244,9 @@ display_externally_modified_notification (GeditTab *tab)
 	g_return_if_fail (location != NULL);
 
 	document_modified = gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc));
-	info_bar = gedit_externally_modified_info_bar_new (location, document_modified);
+	info_bar = tepl_io_error_info_bar_externally_modified (location, document_modified);
 
-	set_info_bar (tab, info_bar, GTK_RESPONSE_OK);
+	set_info_bar (tab, GTK_WIDGET (info_bar));
 
 	g_signal_connect (info_bar,
 			  "response",
@@ -1347,6 +1311,7 @@ gedit_tab_init (GeditTab *tab)
 	GeditDocument *doc;
 	GeditView *view;
 	GtkSourceFile *file;
+	TeplFile *tepl_file;
 
 	tab->state = GEDIT_TAB_STATE_NORMAL;
 
@@ -1377,6 +1342,7 @@ gedit_tab_init (GeditTab *tab)
 	g_object_set_data (G_OBJECT (doc), GEDIT_TAB_KEY, tab);
 
 	file = gedit_document_get_file (doc);
+	tepl_file = tepl_buffer_get_file (TEPL_BUFFER (doc));
 
 	g_signal_connect_object (file,
 				 "notify::location",
@@ -1384,10 +1350,11 @@ gedit_tab_init (GeditTab *tab)
 				 tab,
 				 0);
 
-	g_signal_connect (doc,
-			  "notify::shortname",
-			  G_CALLBACK (document_shortname_notify_handler),
-			  tab);
+	g_signal_connect_object (tepl_file,
+				 "notify::short-name",
+				 G_CALLBACK (document_shortname_notify_handler),
+				 tab,
+				 0);
 
 	g_signal_connect (doc,
 			  "modified_changed",
@@ -1471,7 +1438,7 @@ _gedit_tab_get_name (GeditTab *tab)
 	name = gedit_document_get_short_name_for_display (doc);
 
 	/* Truncate the name so it doesn't get insanely wide. */
-	docname = gedit_utils_str_middle_truncate (name, MAX_DOC_NAME_LENGTH);
+	docname = tepl_utils_str_middle_truncate (name, MAX_DOC_NAME_LENGTH);
 
 	if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)))
 	{
@@ -1504,7 +1471,7 @@ _gedit_tab_get_tooltip (GeditTab *tab)
 	uri = _gedit_document_get_uri_for_display (doc);
 	g_return_val_if_fail (uri != NULL, NULL);
 
-	ruri = 	gedit_utils_replace_home_dir_with_tilde (uri);
+	ruri = 	tepl_utils_replace_home_dir_with_tilde (uri);
 	g_free (uri);
 
 	ruri_markup = g_markup_printf_escaped ("<i>%s</i>", ruri);
@@ -1613,8 +1580,6 @@ _gedit_tab_get_icon (GeditTab *tab)
 	{
 		GdkScreen *screen;
 		GtkIconTheme *theme;
-		GtkIconInfo *info;
-		GtkStyleContext *context;
 		gint icon_size;
 
 		screen = gtk_widget_get_screen (GTK_WIDGET (tab));
@@ -1623,10 +1588,7 @@ _gedit_tab_get_icon (GeditTab *tab)
 
 		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, NULL, &icon_size);
 
-		context = gtk_widget_get_style_context (GTK_WIDGET (tab));
-		info = gtk_icon_theme_lookup_icon (theme, icon_name, icon_size, 0);
-
-		pixbuf = gtk_icon_info_load_symbolic_for_context (info, context, NULL, NULL);
+		pixbuf = gtk_icon_theme_load_icon (theme, icon_name, icon_size, 0, NULL);
 	}
 
 	return pixbuf;
@@ -1804,18 +1766,18 @@ successful_load (GTask *loading_task)
 	if (!gtk_source_file_is_readonly (file) &&
 	    file_already_opened (doc, location))
 	{
-		GtkWidget *info_bar;
+		TeplInfoBar *info_bar;
 
 		set_editable (data->tab, FALSE);
 
-		info_bar = gedit_file_already_open_warning_info_bar_new (location);
+		info_bar = tepl_io_error_info_bar_file_already_open (location);
 
 		g_signal_connect (info_bar,
 				  "response",
 				  G_CALLBACK (file_already_open_warning_info_bar_response),
 				  data->tab);
 
-		set_info_bar (data->tab, info_bar, GTK_RESPONSE_CANCEL);
+		set_info_bar (data->tab, GTK_WIDGET (info_bar));
 	}
 
 	/* When loading from stdin, the contents may not be saved, so set the
@@ -1865,7 +1827,7 @@ load_cb (GtkSourceFileLoader *loader,
 	g_return_if_fail (data->tab->state == GEDIT_TAB_STATE_LOADING ||
 			  data->tab->state == GEDIT_TAB_STATE_REVERTING);
 
-	set_info_bar (data->tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (data->tab, NULL);
 
 	/* Special case creating a named new doc. */
 	create_named_new_doc = (_gedit_document_get_create (doc) &&
@@ -1899,7 +1861,7 @@ load_cb (GtkSourceFileLoader *loader,
 				  G_CALLBACK (io_loading_error_info_bar_response),
 				  loading_task);
 
-		set_info_bar (data->tab, info_bar, GTK_RESPONSE_CANCEL);
+		set_info_bar (data->tab, info_bar);
 
 		if (data->tab->state == GEDIT_TAB_STATE_LOADING)
 		{
@@ -1963,7 +1925,7 @@ load_cb (GtkSourceFileLoader *loader,
 					  loading_task);
 		}
 
-		set_info_bar (data->tab, info_bar, GTK_RESPONSE_CANCEL);
+		set_info_bar (data->tab, info_bar);
 
 		g_error_free (error);
 		return;
@@ -2029,48 +1991,6 @@ get_candidate_encodings (GeditTab *tab)
 	return candidates;
 }
 
-static void
-continue_loading(GTask     *loading_task)
-{
-	LoaderData *data = g_task_get_task_data (loading_task);
-	// Pre loader
-	gtk_source_file_loader_load_async (data->loader,
-					   G_PRIORITY_DEFAULT,
-					   g_task_get_cancellable (loading_task),
-					   (GFileProgressCallback) loader_progress_cb,
-					   loading_task,
-					   NULL,
-					   (GAsyncReadyCallback) load_cb,
-					   loading_task);
-}
-
-
-
-static
-guint64 get_file_size (GFile *file)
-{
-	GFileInfo *info;
-	GError *error = NULL;
-	guint64 res = 0;
-
-	if (file == NULL)
-	{
-		return 0;
-	}
-
-	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NONE, NULL, &error);
-
-	if (error)
-	{
-		g_error_free (error);
-		return 0;
-	}
-
-	res = g_file_info_get_size(info);
-	g_object_unref (info);
-	return res;
-}
-
 static void
 launch_loader (GTask                   *loading_task,
 	       const GtkSourceEncoding *encoding)
@@ -2078,8 +1998,6 @@ launch_loader (GTask                   *loading_task,
 	LoaderData *data = g_task_get_task_data (loading_task);
 	GSList *candidate_encodings = NULL;
 	GeditDocument *doc;
-	const int MB = 1024*1024;
-	const int FILE_TOO_LARGE = 100*MB;
 
 	if (encoding != NULL)
 	{
@@ -2105,47 +2023,6 @@ launch_loader (GTask                   *loading_task,
 
 	data->timer = g_timer_new ();
 
-
-	GFile * file = gtk_source_file_loader_get_location (data->loader);
-	guint64 size = get_file_size (file);
-
-	if (size > FILE_TOO_LARGE)
-	{
-		GtkWidget *info_bar;
-		GError *error = NULL;
-		GFile *location = gtk_source_file_loader_get_location (data->loader);
-		set_editable (data->tab, FALSE);
-
-		error = g_error_new_literal (GTK_SOURCE_FILE_LOADER_ERROR, 0, "");
-		error->code = GTK_SOURCE_FILE_LOADER_ERROR_TOO_BIG;
-
-		info_bar = gedit_io_loading_error_info_bar_new (location, NULL, error);
-
-		g_signal_connect (info_bar,
-				  "response",
-				  G_CALLBACK (io_loading_error_info_bar_response),
-				  loading_task);
-
-		set_info_bar (data->tab, info_bar, GTK_RESPONSE_CANCEL);
-
-		if (data->tab->state == GEDIT_TAB_STATE_LOADING)
-		{
-			gtk_widget_show (GTK_WIDGET (data->tab->frame));
-			gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_LOADING_ERROR);
-		}
-		else
-		{
-			gedit_tab_set_state (data->tab, GEDIT_TAB_STATE_REVERTING_ERROR);
-		}
-
-		successful_load (loading_task);
-		gedit_recent_add_document (doc);
-
-		g_error_free (error);
-		return;
-	}
-
-	// Pre loader
 	gtk_source_file_loader_load_async (data->loader,
 					   G_PRIORITY_DEFAULT,
 					   g_task_get_cancellable (loading_task),
@@ -2322,7 +2199,7 @@ revert_async (GeditTab            *tab,
 
 	if (tab->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
 	{
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 	}
 
 	doc = gedit_tab_get_document (tab);
@@ -2374,7 +2251,7 @@ close_printing (GeditTab *tab)
 	g_clear_object (&tab->print_preview);
 
 	/* destroy the info bar */
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 }
@@ -2422,7 +2299,7 @@ save_cb (GtkSourceFileSaver *saver,
 		data->timer = NULL;
 	}
 
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	if (error != NULL)
 	{
@@ -2434,7 +2311,7 @@ save_cb (GtkSourceFileSaver *saver,
 		    error->code == GTK_SOURCE_FILE_SAVER_ERROR_EXTERNALLY_MODIFIED)
 		{
 			/* This error is recoverable */
-			info_bar = gedit_externally_modified_saving_error_info_bar_new (location, error);
+			info_bar = gedit_externally_modified_saving_error_info_bar_new (location);
 			g_return_if_fail (info_bar != NULL);
 
 			g_signal_connect (info_bar,
@@ -2446,12 +2323,12 @@ save_cb (GtkSourceFileSaver *saver,
 			 error->code == G_IO_ERROR_CANT_CREATE_BACKUP)
 		{
 			/* This error is recoverable */
-			info_bar = gedit_no_backup_saving_error_info_bar_new (location, error);
+			info_bar = GTK_WIDGET (tepl_io_error_info_bar_cant_create_backup (location, error));
 			g_return_if_fail (info_bar != NULL);
 
 			g_signal_connect (info_bar,
 					  "response",
-					  G_CALLBACK (no_backup_error_info_bar_response),
+					  G_CALLBACK (cant_create_backup_error_info_bar_response),
 					  saving_task);
 		}
 		else if (error->domain == GTK_SOURCE_FILE_SAVER_ERROR &&
@@ -2460,7 +2337,7 @@ save_cb (GtkSourceFileSaver *saver,
 			/* If we have any invalid char in the document we must warn the user
 			 * as it can make the document useless if it is saved.
 			 */
-			info_bar = gedit_invalid_character_info_bar_new (location);
+			info_bar = GTK_WIDGET (tepl_io_error_info_bar_invalid_characters (location));
 			g_return_if_fail (info_bar != NULL);
 
 			g_signal_connect (info_bar,
@@ -2494,7 +2371,7 @@ save_cb (GtkSourceFileSaver *saver,
 
 			encoding = gtk_source_file_saver_get_encoding (saver);
 
-			info_bar = gedit_conversion_error_while_saving_info_bar_new (location, encoding, error);
+			info_bar = gedit_conversion_error_while_saving_info_bar_new (location, encoding);
 			g_return_if_fail (info_bar != NULL);
 
 			g_signal_connect (info_bar,
@@ -2503,7 +2380,7 @@ save_cb (GtkSourceFileSaver *saver,
 					  saving_task);
 		}
 
-		set_info_bar (tab, info_bar, GTK_RESPONSE_CANCEL);
+		set_info_bar (tab, info_bar);
 	}
 	else
 	{
@@ -2623,7 +2500,7 @@ _gedit_tab_save_async (GeditTab            *tab,
 		/* We already told the user about the external modification:
 		 * hide the message bar and set the save flag.
 		 */
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 		save_flags |= GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME;
 	}
 
@@ -2732,7 +2609,6 @@ _gedit_tab_save_as_async (GeditTab                 *tab,
 	g_return_if_fail (GEDIT_IS_TAB (tab));
 	g_return_if_fail (tab->state == GEDIT_TAB_STATE_NORMAL ||
 	                  tab->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION ||
-	                  tab->state == GEDIT_TAB_STATE_SAVING_ERROR ||
 	                  tab->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW);
 	g_return_if_fail (G_IS_FILE (location));
 	g_return_if_fail (encoding != NULL);
@@ -2760,14 +2636,9 @@ _gedit_tab_save_as_async (GeditTab                 *tab,
 		/* We already told the user about the external modification:
 		 * hide the message bar and set the save flag.
 		 */
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+		set_info_bar (tab, NULL);
 		save_flags |= GTK_SOURCE_FILE_SAVER_FLAGS_IGNORE_MODIFICATION_TIME;
 	}
-	else if (tab->state == GEDIT_TAB_STATE_SAVING_ERROR)
-	{
-		set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
-		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
-	}
 
 	file = gedit_document_get_file (doc);
 
@@ -2848,15 +2719,15 @@ printing_cb (GeditPrintJob       *job,
 	     GeditPrintJobStatus  status,
 	     GeditTab            *tab)
 {
-	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->info_bar));
+	g_return_if_fail (TEPL_IS_PROGRESS_INFO_BAR (tab->info_bar));
 
 	gtk_widget_show (tab->info_bar);
 
-	gedit_progress_info_bar_set_text (GEDIT_PROGRESS_INFO_BAR (tab->info_bar),
-					  gedit_print_job_get_status_string (job));
+	tepl_progress_info_bar_set_text (TEPL_PROGRESS_INFO_BAR (tab->info_bar),
+					 gedit_print_job_get_status_string (job));
 
-	gedit_progress_info_bar_set_fraction (GEDIT_PROGRESS_INFO_BAR (tab->info_bar),
-					      gedit_print_job_get_progress (job));
+	tepl_progress_info_bar_set_fraction (TEPL_PROGRESS_INFO_BAR (tab->info_bar),
+					     gedit_print_job_get_progress (job));
 }
 
 static void
@@ -2937,7 +2808,7 @@ show_preview_cb (GeditPrintJob     *job,
 	g_return_if_fail (tab->print_preview == NULL);
 
 	/* destroy the info bar */
-	set_info_bar (tab, NULL, GTK_RESPONSE_NONE);
+	set_info_bar (tab, NULL);
 
 	tab->print_preview = GTK_WIDGET (preview);
 	g_object_ref_sink (tab->print_preview);
@@ -2966,21 +2837,19 @@ print_cancelled (GtkWidget *bar,
 static void
 add_printing_info_bar (GeditTab *tab)
 {
-	GtkWidget *bar;
+	TeplProgressInfoBar *bar;
 
-	bar = gedit_progress_info_bar_new ("document-print",
-					   "",
-					   TRUE);
+	bar = tepl_progress_info_bar_new ("document-print", NULL, TRUE);
 
 	g_signal_connect (bar,
 			  "response",
 			  G_CALLBACK (print_cancelled),
 			  tab);
 
-	set_info_bar (tab, bar, GTK_RESPONSE_NONE);
+	set_info_bar (tab, GTK_WIDGET (bar));
 
 	/* hide until we start printing */
-	gtk_widget_hide (bar);
+	gtk_widget_hide (GTK_WIDGET (bar));
 }
 
 void
@@ -3191,7 +3060,7 @@ gedit_tab_set_info_bar (GeditTab  *tab,
 	g_return_if_fail (info_bar == NULL || GTK_IS_WIDGET (info_bar));
 
 	/* FIXME: this can cause problems with the tab state machine */
-	set_info_bar (tab, info_bar, GTK_RESPONSE_NONE);
+	set_info_bar (tab, info_bar);
 }
 
 GeditViewFrame *
